@@ -27,7 +27,7 @@ from ..index import index_connect
 from ..drivers import new_datasource
 
 
-class TerminateCurrentLoad(Exception):
+class TerminateCurrentLoad(Exception):  # noqa: N818
     """ This exception is raised by user code from `progress_cbk`
         to terminate currently running `.load`
     """
@@ -126,16 +126,16 @@ class Datacube(object):
 
         # Optionally compute dataset count for each product and add to row/cols
         # Product lists are sorted by product name to ensure 1:1 match
-        if dataset_count:            
-           
+        if dataset_count:
+
             # Load counts
             counts = [(p.name, c) for p, c in self.index.datasets.count_by_product()]
-            
+
             # Sort both rows and counts by product name
             from operator import itemgetter
             rows = sorted(rows, key=itemgetter(0))
             counts = sorted(counts, key=itemgetter(0))
-            
+
             # Add sorted count to each existing row
             rows = [row + [count[1]] for row, count in zip(rows, counts)]
             cols = cols + ['dataset_count']
@@ -181,10 +181,11 @@ class Datacube(object):
 
     #: pylint: disable=too-many-arguments, too-many-locals
     def load(self, product=None, measurements=None, output_crs=None, resolution=None, resampling=None,
-         skip_broken_datasets=False, dask_chunks=None, like=None, fuse_func=None, align=None,
-         datasets=None, dataset_predicate=None, progress_cbk=None, **query):
+             skip_broken_datasets=False, dask_chunks=None, like=None, fuse_func=None, align=None,
+             datasets=None, dataset_predicate=None, progress_cbk=None, patch_url=None, **query):
         """
-        Load data as an ``xarray.Dataset`` object.  Each measurement will be a data variable in the :class:`xarray.Dataset`.
+        Load data as an ``xarray.Dataset`` object.
+        Each measurement will be a data variable in the :class:`xarray.Dataset`.
 
         See the `xarray documentation <http://xarray.pydata.org/en/stable/data-structures.html>`_ for usage of the
         :class:`xarray.Dataset` and :class:`xarray.DataArray` objects.
@@ -279,7 +280,8 @@ class Datacube(object):
 
         :param list(str) measurements:
             Measurements name or list of names to be included, as listed in :meth:`list_measurements`.
-            These will be loaded as individual ``xr.DataArray`` variables in the output ``xarray.Dataset`` object.
+            These will be loaded as individual ``xr.DataArray`` variables in
+            the output ``xarray.Dataset`` object.
 
             If a list is specified, the measurements will be returned in the order requested.
             By default all available measurements are included.
@@ -289,8 +291,8 @@ class Datacube(object):
             For example: ``'x', 'y', 'time', 'crs'``.
 
         :param str output_crs:
-            The CRS of the returned data, for example ``EPSG:3577``. If no CRS is supplied, the CRS of the stored data is used
-            if available.
+            The CRS of the returned data, for example ``EPSG:3577``.
+            If no CRS is supplied, the CRS of the stored data is used if available.
 
             This differs from the ``crs`` parameter desribed above, which is used to define the CRS
             of the coordinates in the query itself.
@@ -358,7 +360,8 @@ class Datacube(object):
 
         :param function dataset_predicate:
             Optional. A function that can be passed to restrict loaded datasets. A predicate function should
-            take a :class:`datacube.model.Dataset` object (e.g. as returned from :meth:`find_datasets`) and return a boolean.
+            take a :class:`datacube.model.Dataset` object (e.g. as returned from :meth:`find_datasets`) and
+            return a boolean.
             For example, loaded data could be filtered to January observations only by passing the following
             predicate function that returns True for datasets acquired in January::
 
@@ -373,6 +376,10 @@ class Datacube(object):
             if supplied will be called for every file read with ``files_processed_so_far, total_files``. This is
             only applicable to non-lazy loads, ignored when using dask.
 
+        :param Callable[[str], str], patch_url:
+            if supplied, will be used to patch/sign the url(s), as required to access some commercial archives
+            (e.g. Microsoft Planetary Computer).
+
         :return:
             Requested data in a :class:`xarray.Dataset`
 
@@ -431,7 +438,8 @@ class Datacube(object):
                                 dask_chunks=dask_chunks,
                                 skip_broken_datasets=skip_broken_datasets,
                                 progress_cbk=progress_cbk,
-                                extra_dims=extra_dims)
+                                extra_dims=extra_dims,
+                                patch_url=patch_url)
 
         return result
 
@@ -655,7 +663,8 @@ class Datacube(object):
     @staticmethod
     def _xr_load(sources, geobox, measurements,
                  skip_broken_datasets=False,
-                 progress_cbk=None, extra_dims=None):
+                 progress_cbk=None, extra_dims=None,
+                 patch_url=None):
 
         def mk_cbk(cbk):
             if cbk is None:
@@ -700,7 +709,8 @@ class Datacube(object):
             try:
                 _fuse_measurement(data_slice, datasets, geobox, m,
                                   skip_broken_datasets=skip_broken_datasets,
-                                  progress_cbk=_cbk, extra_dim_index=extra_dim_index)
+                                  progress_cbk=_cbk, extra_dim_index=extra_dim_index,
+                                  patch_url=patch_url)
             except (TerminateCurrentLoad, KeyboardInterrupt):
                 data.attrs['dc_partial_load'] = True
                 return data
@@ -710,7 +720,7 @@ class Datacube(object):
     @staticmethod
     def load_data(sources, geobox, measurements, resampling=None,
                   fuse_func=None, dask_chunks=None, skip_broken_datasets=False,
-                  progress_cbk=None, extra_dims=None,
+                  progress_cbk=None, extra_dims=None, patch_url=None,
                   **extra):
         """
         Load data from :meth:`group_datasets` into an :class:`xarray.Dataset`.
@@ -754,6 +764,9 @@ class Datacube(object):
         :param ExtraDimensions extra_dims:
             A ExtraDimensions describing the any additional dimensions on top of (t, y, x)
 
+        :param Callable[[str], str], patch_url:
+            if supplied, will be used to patch/sign the url(s), as required to access some commercial archives.
+
         :rtype: xarray.Dataset
 
         .. seealso:: :meth:`find_datasets` :meth:`group_datasets`
@@ -761,6 +774,9 @@ class Datacube(object):
         measurements = per_band_load_data_settings(measurements, resampling=resampling, fuse_func=fuse_func)
 
         if dask_chunks is not None:
+            if patch_url is not None:
+                # TODO: url mangler for Dask?
+                raise ValueError("The patch_url arguments is not currently supported for Dask loading.")
             return Datacube._dask_load(sources, geobox, measurements, dask_chunks,
                                        skip_broken_datasets=skip_broken_datasets,
                                        extra_dims=extra_dims)
@@ -768,7 +784,8 @@ class Datacube(object):
             return Datacube._xr_load(sources, geobox, measurements,
                                      skip_broken_datasets=skip_broken_datasets,
                                      progress_cbk=progress_cbk,
-                                     extra_dims=extra_dims)
+                                     extra_dims=extra_dims,
+                                     patch_url=patch_url)
 
     def __str__(self):
         return "Datacube<index={!r}>".format(self.index)
@@ -882,24 +899,29 @@ def select_datasets_inside_polygon(datasets, polygon):
             yield dataset
 
 
-def fuse_lazy(datasets, geobox, measurement, skip_broken_datasets=False, prepend_dims=0, extra_dim_index=None):
+def fuse_lazy(datasets, geobox, measurement,
+              skip_broken_datasets=False, prepend_dims=0, extra_dim_index=None, patch_url=None):
     prepend_shape = (1,) * prepend_dims
     data = numpy.full(geobox.shape, measurement.nodata, dtype=measurement.dtype)
     _fuse_measurement(data, datasets, geobox, measurement,
                       skip_broken_datasets=skip_broken_datasets,
-                      extra_dim_index=extra_dim_index)
+                      extra_dim_index=extra_dim_index,
+                      patch_url=patch_url)
     return data.reshape(prepend_shape + geobox.shape)
 
 
 def _fuse_measurement(dest, datasets, geobox, measurement,
                       skip_broken_datasets=False,
                       progress_cbk=None,
-                      extra_dim_index=None):
+                      extra_dim_index=None,
+                      patch_url=None):
     srcs = []
     for ds in datasets:
         src = None
         with ignore_exceptions_if(skip_broken_datasets):
-            src = new_datasource(BandInfo(ds, measurement.name, extra_dim_index=extra_dim_index))
+            src = new_datasource(
+                BandInfo(ds, measurement.name, extra_dim_index=extra_dim_index, patch_url=patch_url)
+            )
 
         if src is None:
             if not skip_broken_datasets:
