@@ -182,9 +182,15 @@ class PostgresDbAPI(object):
     def in_transaction(self):
         return self._connection.in_transaction()
 
+    def begin(self):
+        self._connection.execute(text('BEGIN'))
+
     def rollback(self):
         self._connection.execute(text('ROLLBACK'))
 
+    def commit(self):
+        self._connection.execute(text('COMMIT'))
+
     def execute(self, command):
         return self._connection.execute(command)
 
