@@ -72,8 +72,7 @@ class DatasetResource(AbstractDatasetResource):
         return (self.has(id_) for id_ in ids_)
 
     def add(self, dataset: Dataset,
-            with_lineage: Optional[bool] = None,
-            **kwargs) -> Dataset:
+            with_lineage: bool = True) -> Dataset:
         if with_lineage is None:
             with_lineage = True
         _LOG.info('indexing %s', dataset.id)
@@ -330,9 +329,9 @@ class DatasetResource(AbstractDatasetResource):
             raise ValueError(f"Unsupported query mode: {mode}")
         ids: Set[DSID] = set()
         if mode == "exact":
-            test: Callable[[str], bool] = lambda l: l == uri
+            test: Callable[[str], bool] = lambda l: l == uri  # noqa: E741
         else:
-            test = lambda l: l.startswith(uri)
+            test = lambda l: l.startswith(uri)  # noqa: E731
         for id_, locs in self.locations.items():
             for loc in locs:
                 if test(loc):
@@ -401,9 +400,10 @@ class DatasetResource(AbstractDatasetResource):
             if not product_queries:
                 raise ValueError(f"No products match source filter: {source_filter}")
             if len(product_queries) > 1:
-                raise RuntimeError(f"Multiproduct source_filters are not supported. Try adding 'product' field.")
+                raise RuntimeError("Multiproduct source_filters are not supported. Try adding 'product' field.")
             source_queries, source_product = product_queries[0]
-            source_exprs = tuple(fields.to_expressions(source_product.metadata_type.dataset_fields.get, **source_queries))
+            source_exprs = tuple(fields.to_expressions(source_product.metadata_type.dataset_fields.get,
+                                                       **source_queries))
         else:
             source_product = None
             source_exprs = ()
@@ -462,10 +462,10 @@ class DatasetResource(AbstractDatasetResource):
             **query: QueryField
     ) -> Iterable[Dataset]:
         return cast(Iterable[Dataset], self._search(
-                return_format=self.RET_FORMAT_DATASETS,
-                limit=limit,
-                source_filter=source_filter,
-                **query)
+                    return_format=self.RET_FORMAT_DATASETS,
+                    limit=limit,
+                    source_filter=source_filter,
+                    **query)
         )
 
     def _search_grouped(
@@ -475,10 +475,10 @@ class DatasetResource(AbstractDatasetResource):
             **query: QueryField
     ) -> Iterable[Tuple[Iterable[Dataset], Product]]:
         return cast(Iterable[Tuple[Iterable[Dataset], Product]], self._search(
-                return_format=self.RET_FORMAT_PRODUCT_GROUPED,
-                limit=limit,
-                source_filter=source_filter,
-                **query)
+                    return_format=self.RET_FORMAT_PRODUCT_GROUPED,
+                    limit=limit,
+                    source_filter=source_filter,
+                    **query)
         )
 
     def _get_prod_queries(self, **query: QueryField) -> Iterable[Tuple[Mapping[str, QueryField], Product]]:
@@ -503,8 +503,8 @@ class DatasetResource(AbstractDatasetResource):
         for ds in self.search(limit=limit, **query):  # type: ignore[arg-type]
             ds_fields = get_dataset_fields(ds.type.metadata_type.definition)
             result_vals = {
-                 fn: ds_fields[fn].extract(ds.metadata_doc)  # type: ignore[attr-defined]
-                 for fn in field_names
+                fn: ds_fields[fn].extract(ds.metadata_doc)  # type: ignore[attr-defined]
+                for fn in field_names
             }
             yield result_type(**result_vals)
 
@@ -516,8 +516,8 @@ class DatasetResource(AbstractDatasetResource):
             yield (prod, len(list(datasets)))
 
     def count_by_product_through_time(self,
-            period: str,
-            **query: QueryField
+                                      period: str,
+                                      **query: QueryField
     ) -> Iterable[
         Tuple[
             Product,
@@ -543,6 +543,7 @@ class DatasetResource(AbstractDatasetResource):
         if precision <= 0:
             raise ValueError('Invalid period string. Must specify a natural number of days, weeks, months or years')
         unit = match.group("unit")
+
         def next_period(prev: datetime.datetime) -> datetime.datetime:
             if unit == 'day':
                 return prev + datetime.timedelta(days=precision)
@@ -598,7 +599,7 @@ class DatasetResource(AbstractDatasetResource):
             ],
         ]
     ]:
-        YieldType = Tuple[Product, Iterable[Tuple[Range, int]]]
+        YieldType = Tuple[Product, Iterable[Tuple[Range, int]]]  # noqa: N806
         query = dict(query)
         try:
             start, end = cast(Range, query.pop('time'))
@@ -665,7 +666,7 @@ class DatasetResource(AbstractDatasetResource):
                 min_time = dsmin
             if max_time is None or dsmax > max_time:
                 max_time = dsmax
-        return (cast(datetime.datetime, min_time), cast(datetime.datetime,max_time))
+        return (cast(datetime.datetime, min_time), cast(datetime.datetime, max_time))
 
     # pylint: disable=redefined-outer-name
     def search_returning_datasets_light(
@@ -679,6 +680,7 @@ class DatasetResource(AbstractDatasetResource):
             custom_fields = build_custom_fields(custom_offsets)
         else:
             custom_fields = {}
+
         def make_ds_light(ds: Dataset) -> Tuple:
             fields = {
                 fname: ds.metadata_type.dataset_fields[fname]
@@ -718,3 +720,6 @@ class DatasetResource(AbstractDatasetResource):
             indexed_time=datetime.datetime.now() if for_save and orig.indexed_time is None else orig.indexed_time,
             archived_time=None if for_save else orig.archived_time
         )
+
+    def spatial_extent(self, ids, crs=None):
+        return None
