@@ -9,19 +9,21 @@ import json
 import logging
 import warnings
 from collections import namedtuple
-from typing import Iterable, Tuple, Union, List, Optional
+from typing import Iterable, Union, List, Optional
 from uuid import UUID
 
 from sqlalchemy import select, func
 
 from datacube.drivers.postgis._fields import SimpleDocField, DateDocField
-from datacube.drivers.postgis._schema import DATASET
-from datacube.index.abstract import AbstractDatasetResource, DSID
-from datacube.model import Dataset, DatasetType
+from datacube.drivers.postgis._schema import Dataset as SQLDataset
+from datacube.index.abstract import AbstractDatasetResource, DatasetSpatialMixin, DSID
+from datacube.index.postgis._transaction import IndexResourceAddIn
+from datacube.model import Dataset, Product
 from datacube.model.fields import Field
 from datacube.model.utils import flatten_datasets
-from datacube.utils import jsonify_document, _readable_offset, changes, cached_property
+from datacube.utils import jsonify_document, _readable_offset, changes
 from datacube.utils.changes import get_doc_changes
+from datacube.utils.geometry import CRS, Geometry
 from datacube.index import fields
 
 _LOG = logging.getLogger(__name__)
@@ -30,43 +32,22 @@ _LOG = logging.getLogger(__name__)
 # It's a public api, so we can't reorganise old methods.
 # pylint: disable=too-many-public-methods, too-many-lines
 
-class DatasetSpatialMixin(object):
-    __slots__ = ()
 
-    @property
-    def _gs(self):
-        return self.grid_spatial
-
-    @property
-    def crs(self):
-        return Dataset.crs.__get__(self)
-
-    @cached_property
-    def extent(self):
-        return Dataset.extent.func(self)
-
-    @property
-    def transform(self):
-        return Dataset.transform.__get__(self)
-
-    @property
-    def bounds(self):
-        return Dataset.bounds.__get__(self)
-
-
-class DatasetResource(AbstractDatasetResource):
+class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
     """
     :type _db: datacube.drivers.postgis._connections.PostgresDb
     :type types: datacube.index._products.ProductResource
     """
 
-    def __init__(self, db, dataset_type_resource):
+    def __init__(self, db, index):
         """
         :type db: datacube.drivers.postgis._connections.PostgresDb
-        :type dataset_type_resource: datacube.index._products.ProductResource
+        :type product_resource: datacube.index._products.ProductResource
         """
         self._db = db
-        self.types = dataset_type_resource
+        self._index = index
+        self.types = self._index.products   # types is a compatibility alias for products.
+        self.products = self._index.products
 
     def get(self, id_: Union[str, UUID], include_sources=False):
         """
@@ -79,7 +60,7 @@ class DatasetResource(AbstractDatasetResource):
         if isinstance(id_, str):
             id_ = UUID(id_)
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             if not include_sources:
                 dataset = connection.get_dataset(id_)
                 return self._make(dataset, full_info=True) if dataset else None
@@ -108,7 +89,7 @@ class DatasetResource(AbstractDatasetResource):
 
         ids = [to_uuid(i) for i in ids]
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             rows = connection.get_datasets(ids)
             return [self._make(r, full_info=True) for r in rows]
 
@@ -121,7 +102,7 @@ class DatasetResource(AbstractDatasetResource):
         """
         if not isinstance(id_, UUID):
             id_ = UUID(id_)
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return [
                 self._make(result, full_info=True)
                 for result in connection.get_derived_datasets(id_)
@@ -134,7 +115,7 @@ class DatasetResource(AbstractDatasetResource):
         :param typing.Union[UUID, str] id_: dataset id
         :rtype: bool
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return connection.contains_dataset(id_)
 
     def bulk_has(self, ids_):
@@ -147,58 +128,53 @@ class DatasetResource(AbstractDatasetResource):
 
         :rtype: [bool]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             existing = set(connection.datasets_intersection(ids_))
 
         return [x in existing for x in
                 map((lambda x: UUID(x) if isinstance(x, str) else x), ids_)]
 
     def add(self, dataset: Dataset,
-            with_lineage: Optional[bool] = None,
-            **kwargs) -> Dataset:
+            with_lineage: bool = True) -> Dataset:
         """
         Add ``dataset`` to the index. No-op if it is already present.
 
         :param dataset: dataset to add
 
         :param with_lineage:
-           - ``True|None`` attempt adding lineage datasets if missing
+           - ``True (default)`` attempt adding lineage datasets if missing
            - ``False`` record lineage relations, but do not attempt
              adding lineage datasets to the db
 
-        :param kwargs: only used to support deprecated behaviour
         :rtype: Dataset
         """
 
+        sp_crses = self._db.spatial_indexes()
+
         def process_bunch(dss, main_ds, transaction):
             edges = []
-
-            # First insert all new datasets
+            dsids_for_spatial_indexing = []
+            # 1: Loop over datasets
             for ds in dss:
+                # 1a. insert (if not already exists)
                 is_new = transaction.insert_dataset(ds.metadata_doc_without_lineage(), ds.id, ds.type.id)
                 sources = ds.sources
+                # 1b. Build edge graph for new datasets
                 if is_new and sources is not None:
                     edges.extend((name, ds.id, src.id)
                                  for name, src in sources.items())
-
-            # Second insert lineage graph edges
+                # 1c. Prepare spatial index extents
+                if is_new:
+                    dsids_for_spatial_indexing.append(ds.id)
+            # 2: insert lineage graph edges
             for ee in edges:
                 transaction.insert_dataset_source(*ee)
-
+            # 3: insert spatial indexes
+            transaction.update_spindex(dsids=dsids_for_spatial_indexing)
             # Finally update location for top-level dataset only
             if main_ds.uris is not None:
                 self._ensure_new_locations(main_ds, transaction=transaction)
 
-        if with_lineage is None:
-            policy = kwargs.pop('sources_policy', None)
-            if policy is not None:
-                _LOG.debug('Use of sources_policy is deprecated')
-                with_lineage = (policy != "skip")
-                if policy == 'verify':
-                    _LOG.debug('Verify is no longer done inside add')
-            else:
-                with_lineage = True
-
         _LOG.info('Indexing %s', dataset.id)
 
         if with_lineage:
@@ -219,12 +195,12 @@ class DatasetResource(AbstractDatasetResource):
 
             dss = [dataset]
 
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             process_bunch(dss, dataset, transaction)
 
         return dataset
 
-    def search_product_duplicates(self, product: DatasetType, *args):
+    def search_product_duplicates(self, product: Product, *args):
         """
         Find dataset ids who have duplicates of the given set of field names.
 
@@ -244,7 +220,7 @@ class DatasetResource(AbstractDatasetResource):
 
         expressions = [product.metadata_type.dataset_fields.get('product') == product.name]
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             for record in connection.get_duplicates(group_fields, expressions):
                 dataset_ids = set(record[0])
                 grouped_fields = tuple(record[1:])
@@ -259,6 +235,8 @@ class DatasetResource(AbstractDatasetResource):
         :rtype: bool,list[change],list[change]
         """
         need_sources = dataset.sources is not None
+        # TODO: Source retrieval is broken.
+        need_sources = False
         existing = self.get(dataset.id, include_sources=need_sources)
         if not existing:
             raise ValueError('Unknown dataset %s, cannot update – did you intend to add it?' % dataset.id)
@@ -312,7 +290,7 @@ class DatasetResource(AbstractDatasetResource):
         _LOG.info("Updating dataset %s", dataset.id)
 
         product = self.types.get_by_name(dataset.type.name)
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             if not transaction.update_dataset(dataset.metadata_doc_without_lineage(), dataset.id, product.id):
                 raise ValueError("Failed to update dataset %s..." % dataset.id)
 
@@ -331,7 +309,7 @@ class DatasetResource(AbstractDatasetResource):
         # front of a stack
         for uri in new_uris[::-1]:
             if transaction is None:
-                with self._db.begin() as tr:
+                with self._db_connection(transaction=True) as tr:
                     insert_one(uri, tr)
             else:
                 insert_one(uri, transaction)
@@ -342,7 +320,7 @@ class DatasetResource(AbstractDatasetResource):
 
         :param Iterable[UUID] ids: list of dataset ids to archive
         """
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             for id_ in ids:
                 transaction.archive_dataset(id_)
 
@@ -352,7 +330,7 @@ class DatasetResource(AbstractDatasetResource):
 
         :param Iterable[UUID] ids: list of dataset ids to restore
         """
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             for id_ in ids:
                 transaction.restore_dataset(id_)
 
@@ -362,7 +340,7 @@ class DatasetResource(AbstractDatasetResource):
 
         :param ids: iterable of dataset ids to purge
         """
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             for id_ in ids:
                 transaction.delete_dataset(id_)
 
@@ -376,7 +354,7 @@ class DatasetResource(AbstractDatasetResource):
         :param archived:
         :rtype: list[UUID]
         """
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             return [dsid[0] for dsid in transaction.all_dataset_ids(archived)]
 
     def get_field_names(self, product_name=None):
@@ -403,7 +381,7 @@ class DatasetResource(AbstractDatasetResource):
         :param typing.Union[UUID, str] id_: dataset id
         :rtype: list[str]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return connection.get_locations(id_)
 
     def get_archived_locations(self, id_):
@@ -413,7 +391,7 @@ class DatasetResource(AbstractDatasetResource):
         :param typing.Union[UUID, str] id_: dataset id
         :rtype: list[str]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return [uri for uri, archived_dt in connection.get_archived_locations(id_)]
 
     def get_archived_location_times(self, id_):
@@ -423,7 +401,7 @@ class DatasetResource(AbstractDatasetResource):
         :param typing.Union[UUID, str] id_: dataset id
         :rtype: List[Tuple[str, datetime.datetime]]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return list(connection.get_archived_locations(id_))
 
     def add_location(self, id_, uri):
@@ -438,7 +416,7 @@ class DatasetResource(AbstractDatasetResource):
             warnings.warn("Cannot add empty uri. (dataset %s)" % id_)
             return False
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return connection.insert_dataset_location(id_, uri)
 
     def get_datasets_for_location(self, uri, mode=None):
@@ -449,7 +427,7 @@ class DatasetResource(AbstractDatasetResource):
         :param str mode: 'exact', 'prefix' or None (to guess)
         :return:
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return (self._make(row) for row in connection.get_datasets_for_location(uri, mode=mode))
 
     def remove_location(self, id_, uri):
@@ -460,7 +438,7 @@ class DatasetResource(AbstractDatasetResource):
         :param str uri: fully qualified uri
         :returns bool: Was one removed?
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             was_removed = connection.remove_location(id_, uri)
             return was_removed
 
@@ -472,7 +450,7 @@ class DatasetResource(AbstractDatasetResource):
         :param str uri: fully qualified uri
         :return bool: location was able to be archived
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             was_archived = connection.archive_location(id_, uri)
             return was_archived
 
@@ -484,7 +462,7 @@ class DatasetResource(AbstractDatasetResource):
         :param str uri: fully qualified uri
         :return bool: location was able to be restored
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             was_restored = connection.restore_location(id_, uri)
             return was_restored
 
@@ -499,7 +477,7 @@ class DatasetResource(AbstractDatasetResource):
         else:
             uris = []
 
-        product = product or self.types.get(dataset_res.dataset_type_ref)
+        product = product or self.types.get(dataset_res.product_ref)
 
         return Dataset(
             type_=product,
@@ -525,7 +503,7 @@ class DatasetResource(AbstractDatasetResource):
         :param dict metadata:
         :rtype: list[Dataset]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             for dataset in self._make_many(connection.search_datasets_by_metadata(metadata)):
                 yield dataset
 
@@ -548,7 +526,7 @@ class DatasetResource(AbstractDatasetResource):
         Perform a search, returning datasets grouped by product type.
 
         :param dict[str,str|float|datacube.model.Range] query:
-        :rtype: __generator[(DatasetType,  __generator[Dataset])]]
+        :rtype: __generator[(Product,  __generator[Dataset])]]
         """
         for product, datasets in self._do_search_by_product(query):
             yield product, self._make_many(datasets, product)
@@ -572,7 +550,6 @@ class DatasetResource(AbstractDatasetResource):
                                                      return_fields=True,
                                                      select_field_names=field_names,
                                                      limit=limit):
-
             for columns in results:
                 yield result_type(*columns)
 
@@ -596,7 +573,7 @@ class DatasetResource(AbstractDatasetResource):
 
         :param dict[str,str|float|datacube.model.Range] query:
         :returns: Sequence of (product, count)
-        :rtype: __generator[(DatasetType,  int)]]
+        :rtype: __generator[(Product,  int)]]
         """
         return self._do_count_by_product(query)
 
@@ -608,7 +585,7 @@ class DatasetResource(AbstractDatasetResource):
         :param dict[str,str|float|datacube.model.Range] query:
         :param str period: Time range for each slice: '1 month', '1 day' etc.
         :returns: For each matching product type, a list of time ranges and their count.
-        :rtype: __generator[(DatasetType, list[(datetime.datetime, datetime.datetime), int)]]
+        :rtype: __generator[(Product, list[(datetime.datetime, datetime.datetime), int)]]
         """
         return self._do_time_count(period, query)
 
@@ -626,7 +603,7 @@ class DatasetResource(AbstractDatasetResource):
         """
         return next(self._do_time_count(period, query, ensure_single=True))[1]
 
-    def _get_dataset_types(self, q):
+    def _get_products(self, q):
         types = set()
         if 'product' in q.keys():
             types.add(self.types.get_by_name(q['product']))
@@ -640,27 +617,15 @@ class DatasetResource(AbstractDatasetResource):
 
     def _get_product_queries(self, query):
         for product, q in self.types.search_robust(**query):
-            q['dataset_type_id'] = product.id
+            q['product_id'] = product.id
             yield q, product
 
     # pylint: disable=too-many-locals
     def _do_search_by_product(self, query, return_fields=False, select_field_names=None,
                               with_source_ids=False, source_filter=None,
                               limit=None):
-        if source_filter:
-            product_queries = list(self._get_product_queries(source_filter))
-            if not product_queries:
-                # No products match our source filter, so there will be no search results regardless.
-                raise ValueError('No products match source filter: ' % source_filter)
-            if len(product_queries) > 1:
-                raise RuntimeError("Multi-product source filters are not supported. Try adding 'product' field")
-
-            source_queries, source_product = product_queries[0]
-            dataset_fields = source_product.metadata_type.dataset_fields
-            source_exprs = tuple(fields.to_expressions(dataset_fields.get, **source_queries))
-        else:
-            source_exprs = None
-
+        assert not with_source_ids
+        assert source_filter is None
         product_queries = list(self._get_product_queries(query))
         if not product_queries:
             product = query.get('product', None)
@@ -670,6 +635,12 @@ class DatasetResource(AbstractDatasetResource):
                 raise ValueError(f"No such product: {product}")
 
         for q, product in product_queries:
+            _LOG.warning("Querying product %s", product)
+            # Extract Geospatial search geometry
+            geom = self._extract_geom_from_query(q)
+            assert "lat" not in q
+            assert "lon" not in q
+
             dataset_fields = product.metadata_type.dataset_fields
             query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
             select_fields = None
@@ -681,14 +652,14 @@ class DatasetResource(AbstractDatasetResource):
                 else:
                     select_fields = tuple(dataset_fields[field_name]
                                           for field_name in select_field_names)
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 yield (product,
                        connection.search_datasets(
                            query_exprs,
-                           source_exprs,
                            select_fields=select_fields,
                            limit=limit,
-                           with_source_ids=with_source_ids
+                           with_source_ids=with_source_ids,
+                           geom=geom
                        ))
 
     def _do_count_by_product(self, query):
@@ -697,7 +668,7 @@ class DatasetResource(AbstractDatasetResource):
         for q, product in product_queries:
             dataset_fields = product.metadata_type.dataset_fields
             query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 count = connection.count_datasets(query_exprs)
             if count > 0:
                 yield product, count
@@ -722,7 +693,7 @@ class DatasetResource(AbstractDatasetResource):
         for q, product in product_queries:
             dataset_fields = product.metadata_type.dataset_fields
             query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 yield product, list(connection.count_datasets_through_time(
                     start,
                     end,
@@ -740,7 +711,9 @@ class DatasetResource(AbstractDatasetResource):
         """
         for _, results in self._do_search_by_product(query, return_fields=True):
             for columns in results:
-                yield dict(columns)
+                output = dict(columns)
+                _LOG.warning("search results: %s (%s)", output["id"], output["product"])
+                yield output
 
     def get_product_time_bounds(self, product: str):
         """
@@ -755,24 +728,24 @@ class DatasetResource(AbstractDatasetResource):
 
         time_min = DateDocField('aquisition_time_min',
                                 'Min of time when dataset was acquired',
-                                DATASET.c.metadata,
+                                SQLDataset.metadata_doc,
                                 False,  # is it indexed
                                 offset=min_offset,
                                 selection='least')
 
         time_max = DateDocField('aquisition_time_max',
                                 'Max of time when dataset was acquired',
-                                DATASET.c.metadata,
+                                SQLDataset.metadata_doc,
                                 False,  # is it indexed
                                 offset=max_offset,
                                 selection='greatest')
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             result = connection.execute(
                 select(
                     [func.min(time_min.alchemy_expression), func.max(time_max.alchemy_expression)]
                 ).where(
-                    DATASET.c.dataset_type_ref == product.id
+                    SQLDataset.product_ref == product.id
                 )
             ).first()
 
@@ -821,7 +794,7 @@ class DatasetResource(AbstractDatasetResource):
                 class DatasetLight(result_type):  # type: ignore
                     __slots__ = ()
 
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 results = connection.search_unique_datasets(
                     query_exprs,
                     select_fields=select_fields,
@@ -859,13 +832,13 @@ class DatasetResource(AbstractDatasetResource):
                     grid_spatial = dataset_section.get('grid_spatial')
                     if grid_spatial:
                         select_fields.append(SimpleDocField(
-                            'grid_spatial', 'grid_spatial', DATASET.c.metadata,
+                            'grid_spatial', 'grid_spatial', SQLDataset.metadata_doc,
                             False,
                             offset=grid_spatial
                         ))
                 elif custom_offsets and field_name in custom_offsets:
                     select_fields.append(SimpleDocField(
-                        field_name, field_name, DATASET.c.metadata,
+                        field_name, field_name, SQLDataset.metadata_doc,
                         False,
                         offset=custom_offsets[field_name]
                     ))
@@ -912,9 +885,13 @@ class DatasetResource(AbstractDatasetResource):
         for key in custom_query:
             # for now we assume all custom query fields are SimpleDocFields
             custom_field = SimpleDocField(
-                custom_query[key], custom_query[key], DATASET.c.metadata,
+                custom_query[key], custom_query[key], Dataset.metadata,
                 False, offset=custom_offsets[key]
             )
             custom_exprs.append(fields.as_expression(custom_field, custom_query[key]))
 
         return custom_exprs
+
+    def spatial_extent(self, ids: Iterable[DSID], crs: CRS = CRS("EPSG:4326")) -> Optional[Geometry]:
+        with self._db_connection() as connection:
+            return connection.spatial_extent(ids, crs)
