@@ -1,6 +1,6 @@
 # This file is part of the Open Data Cube, see https://opendatacube.org for more information
 #
-# Copyright (c) 2015-2020 ODC Contributors
+# Copyright (c) 2015-2022 ODC Contributors
 # SPDX-License-Identifier: Apache-2.0
 """
 API for dataset indexing, access and search.
@@ -9,7 +9,7 @@ import json
 import logging
 import warnings
 from collections import namedtuple
-from typing import Iterable, Tuple, Union, List, Optional
+from typing import Iterable, Union, List
 from uuid import UUID
 
 from sqlalchemy import select, func
@@ -17,6 +17,7 @@ from sqlalchemy import select, func
 from datacube.drivers.postgres._fields import SimpleDocField, DateDocField
 from datacube.drivers.postgres._schema import DATASET
 from datacube.index.abstract import AbstractDatasetResource, DatasetSpatialMixin, DSID
+from datacube.index.postgres._transaction import IndexResourceAddIn
 from datacube.model import Dataset, DatasetType
 from datacube.model.fields import Field
 from datacube.model.utils import flatten_datasets
@@ -29,21 +30,20 @@ _LOG = logging.getLogger(__name__)
 
 # It's a public api, so we can't reorganise old methods.
 # pylint: disable=too-many-public-methods, too-many-lines
-
-
-class DatasetResource(AbstractDatasetResource):
+class DatasetResource(AbstractDatasetResource, IndexResourceAddIn):
     """
     :type _db: datacube.drivers.postgres._connections.PostgresDb
     :type types: datacube.index._products.ProductResource
     """
 
-    def __init__(self, db, dataset_type_resource):
+    def __init__(self, db, index):
         """
         :type db: datacube.drivers.postgres._connections.PostgresDb
         :type dataset_type_resource: datacube.index._products.ProductResource
         """
         self._db = db
-        self.types = dataset_type_resource
+        self._index = index
+        self.types = self._index.products
 
     def get(self, id_: Union[str, UUID], include_sources=False):
         """
@@ -56,7 +56,7 @@ class DatasetResource(AbstractDatasetResource):
         if isinstance(id_, str):
             id_ = UUID(id_)
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             if not include_sources:
                 dataset = connection.get_dataset(id_)
                 return self._make(dataset, full_info=True) if dataset else None
@@ -85,7 +85,7 @@ class DatasetResource(AbstractDatasetResource):
 
         ids = [to_uuid(i) for i in ids]
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             rows = connection.get_datasets(ids)
             return [self._make(r, full_info=True) for r in rows]
 
@@ -98,7 +98,7 @@ class DatasetResource(AbstractDatasetResource):
         """
         if not isinstance(id_, UUID):
             id_ = UUID(id_)
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return [
                 self._make(result, full_info=True)
                 for result in connection.get_derived_datasets(id_)
@@ -111,7 +111,7 @@ class DatasetResource(AbstractDatasetResource):
         :param typing.Union[UUID, str] id_: dataset id
         :rtype: bool
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return connection.contains_dataset(id_)
 
     def bulk_has(self, ids_):
@@ -124,26 +124,24 @@ class DatasetResource(AbstractDatasetResource):
 
         :rtype: [bool]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             existing = set(connection.datasets_intersection(ids_))
 
         return [x in existing for x in
                 map((lambda x: UUID(x) if isinstance(x, str) else x), ids_)]
 
     def add(self, dataset: Dataset,
-            with_lineage: Optional[bool] = None,
-            **kwargs) -> Dataset:
+            with_lineage: bool = True) -> Dataset:
         """
         Add ``dataset`` to the index. No-op if it is already present.
 
         :param dataset: dataset to add
 
         :param with_lineage:
-           - ``True|None`` attempt adding lineage datasets if missing
+           - ``True (default)`` attempt adding lineage datasets if missing
            - ``False`` record lineage relations, but do not attempt
              adding lineage datasets to the db
 
-        :param kwargs: only used to support deprecated behaviour
         :rtype: Dataset
         """
 
@@ -166,16 +164,6 @@ class DatasetResource(AbstractDatasetResource):
             if main_ds.uris is not None:
                 self._ensure_new_locations(main_ds, transaction=transaction)
 
-        if with_lineage is None:
-            policy = kwargs.pop('sources_policy', None)
-            if policy is not None:
-                _LOG.debug('Use of sources_policy is deprecated')
-                with_lineage = (policy != "skip")
-                if policy == 'verify':
-                    _LOG.debug('Verify is no longer done inside add')
-            else:
-                with_lineage = True
-
         _LOG.info('Indexing %s', dataset.id)
 
         if with_lineage:
@@ -196,7 +184,7 @@ class DatasetResource(AbstractDatasetResource):
 
             dss = [dataset]
 
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             process_bunch(dss, dataset, transaction)
 
         return dataset
@@ -221,7 +209,7 @@ class DatasetResource(AbstractDatasetResource):
 
         expressions = [product.metadata_type.dataset_fields.get('product') == product.name]
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             for record in connection.get_duplicates(group_fields, expressions):
                 dataset_ids = set(record[0])
                 grouped_fields = tuple(record[1:])
@@ -289,7 +277,7 @@ class DatasetResource(AbstractDatasetResource):
         _LOG.info("Updating dataset %s", dataset.id)
 
         product = self.types.get_by_name(dataset.type.name)
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             if not transaction.update_dataset(dataset.metadata_doc_without_lineage(), dataset.id, product.id):
                 raise ValueError("Failed to update dataset %s..." % dataset.id)
 
@@ -308,7 +296,7 @@ class DatasetResource(AbstractDatasetResource):
         # front of a stack
         for uri in new_uris[::-1]:
             if transaction is None:
-                with self._db.begin() as tr:
+                with self._db_connection(transaction=True) as tr:
                     insert_one(uri, tr)
             else:
                 insert_one(uri, transaction)
@@ -319,7 +307,7 @@ class DatasetResource(AbstractDatasetResource):
 
         :param Iterable[UUID] ids: list of dataset ids to archive
         """
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             for id_ in ids:
                 transaction.archive_dataset(id_)
 
@@ -329,7 +317,7 @@ class DatasetResource(AbstractDatasetResource):
 
         :param Iterable[UUID] ids: list of dataset ids to restore
         """
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             for id_ in ids:
                 transaction.restore_dataset(id_)
 
@@ -339,7 +327,7 @@ class DatasetResource(AbstractDatasetResource):
 
         :param ids: iterable of dataset ids to purge
         """
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             for id_ in ids:
                 transaction.delete_dataset(id_)
 
@@ -353,7 +341,7 @@ class DatasetResource(AbstractDatasetResource):
         :param archived:
         :rtype: list[UUID]
         """
-        with self._db.begin() as transaction:
+        with self._db_connection(transaction=True) as transaction:
             return [dsid[0] for dsid in transaction.all_dataset_ids(archived)]
 
     def get_field_names(self, product_name=None):
@@ -380,7 +368,7 @@ class DatasetResource(AbstractDatasetResource):
         :param typing.Union[UUID, str] id_: dataset id
         :rtype: list[str]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return connection.get_locations(id_)
 
     def get_archived_locations(self, id_):
@@ -390,7 +378,7 @@ class DatasetResource(AbstractDatasetResource):
         :param typing.Union[UUID, str] id_: dataset id
         :rtype: list[str]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return [uri for uri, archived_dt in connection.get_archived_locations(id_)]
 
     def get_archived_location_times(self, id_):
@@ -400,7 +388,7 @@ class DatasetResource(AbstractDatasetResource):
         :param typing.Union[UUID, str] id_: dataset id
         :rtype: List[Tuple[str, datetime.datetime]]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return list(connection.get_archived_locations(id_))
 
     def add_location(self, id_, uri):
@@ -415,7 +403,7 @@ class DatasetResource(AbstractDatasetResource):
             warnings.warn("Cannot add empty uri. (dataset %s)" % id_)
             return False
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return connection.insert_dataset_location(id_, uri)
 
     def get_datasets_for_location(self, uri, mode=None):
@@ -426,7 +414,7 @@ class DatasetResource(AbstractDatasetResource):
         :param str mode: 'exact', 'prefix' or None (to guess)
         :return:
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             return (self._make(row) for row in connection.get_datasets_for_location(uri, mode=mode))
 
     def remove_location(self, id_, uri):
@@ -437,7 +425,7 @@ class DatasetResource(AbstractDatasetResource):
         :param str uri: fully qualified uri
         :returns bool: Was one removed?
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             was_removed = connection.remove_location(id_, uri)
             return was_removed
 
@@ -449,7 +437,7 @@ class DatasetResource(AbstractDatasetResource):
         :param str uri: fully qualified uri
         :return bool: location was able to be archived
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             was_archived = connection.archive_location(id_, uri)
             return was_archived
 
@@ -461,7 +449,7 @@ class DatasetResource(AbstractDatasetResource):
         :param str uri: fully qualified uri
         :return bool: location was able to be restored
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             was_restored = connection.restore_location(id_, uri)
             return was_restored
 
@@ -502,7 +490,7 @@ class DatasetResource(AbstractDatasetResource):
         :param dict metadata:
         :rtype: list[Dataset]
         """
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             for dataset in self._make_many(connection.search_datasets_by_metadata(metadata)):
                 yield dataset
 
@@ -658,7 +646,7 @@ class DatasetResource(AbstractDatasetResource):
                 else:
                     select_fields = tuple(dataset_fields[field_name]
                                           for field_name in select_field_names)
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 yield (product,
                        connection.search_datasets(
                            query_exprs,
@@ -674,7 +662,7 @@ class DatasetResource(AbstractDatasetResource):
         for q, product in product_queries:
             dataset_fields = product.metadata_type.dataset_fields
             query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 count = connection.count_datasets(query_exprs)
             if count > 0:
                 yield product, count
@@ -699,7 +687,7 @@ class DatasetResource(AbstractDatasetResource):
         for q, product in product_queries:
             dataset_fields = product.metadata_type.dataset_fields
             query_exprs = tuple(fields.to_expressions(dataset_fields.get, **q))
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 yield product, list(connection.count_datasets_through_time(
                     start,
                     end,
@@ -744,7 +732,7 @@ class DatasetResource(AbstractDatasetResource):
                                 offset=max_offset,
                                 selection='greatest')
 
-        with self._db.connect() as connection:
+        with self._db_connection() as connection:
             result = connection.execute(
                 select(
                     [func.min(time_min.alchemy_expression), func.max(time_max.alchemy_expression)]
@@ -798,7 +786,7 @@ class DatasetResource(AbstractDatasetResource):
                 class DatasetLight(result_type):  # type: ignore
                     __slots__ = ()
 
-            with self._db.connect() as connection:
+            with self._db_connection() as connection:
                 results = connection.search_unique_datasets(
                     query_exprs,
                     select_fields=select_fields,
@@ -895,3 +883,6 @@ class DatasetResource(AbstractDatasetResource):
             custom_exprs.append(fields.as_expression(custom_field, custom_query[key]))
 
         return custom_exprs
+
+    def spatial_extent(self, ids, crs=None):
+        return None
